# Autogenerated by Astropy-affiliated package astropy_helpers's setup.py on 2015-07-22 07:51:57.579020
from __future__ import unicode_literals
import datetime

version = "1.0.2"
githash = "161773fa72d916c498e0a2a513ecc24460244ac8"


major = 1
minor = 0
bugfix = 2

release = True
timestamp = datetime.datetime(2015, 7, 22, 7, 51, 57, 579020)
debug = False

try:
    from ._compiler import compiler
except ImportError:
    compiler = "unknown"

try:
    from .cython_version import cython_version
except ImportError:
    cython_version = "unknown"
